;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x91d465db1b3d70ef79d16ac21a673681 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x72f0546577e43a672313f7b5a58248d8 . "./code.cg2js/cg-printing.fasl")
(#x0968b12ba7cdf3ec3d875c591763de3b . "./code.cg2js/cg-caret.fasl")
(#xd40920ae6a4718dcfd72e5bbfef9916c . "./code.cg2js/cg-item-list.fasl")
(#x8195acc5b6f4b9d40c871d652cf8cb45 . "./code.cg2js/cg-progress-indicator.fasl")
(#x30c30a0ffc8bfd994781e26f8d1c1ac1 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x34eef2c5f878260c3e8c73f08f42f03c . "./code.cg2js/cg-outline.fasl")
(#x4ac1c866257e94c1261b5bf2fad57b3c . "./code.cg2js/cg-palette.fasl")
(#x1c1ce1936f862e64ed4e50f98aaf2677 . "./code.cg2js/cg-selected-object.fasl")
(#x16c36b35a2936b39862fcee6b9e4674b . "./code.cg2js/cg-status-bar.fasl")
(#x1042229e9cfa3f077fc013ce60b5c184 . "./code.cg2js/cg-wrap-string.fasl")
(#x1f245547e6c8bf5d9ecc3621934a80e8 . "./code.cg2js/cg-fill-texture.fasl")
(#x19e00dd5514000ad8617a9a98df606d0 . "./code.cg2js/cg-layout.fasl")
(#x3a6c170999631183d9f3470f5009fcbb . "./code.cg2js/cg-timer.fasl")
(#xea4c643bb876068468885d123a433bd4 . "./code.cg2js/cg-dialog-item.fasl")
(#xa24312f1a1f0ad3118751947f6bd1b90 . "./code.cg2js/cg-button.fasl")
(#xde203b0c28b4559e16e9b127a867c4c2 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x566dc4b1a8f6b3969062bdb3d693d340 . "./code.cg2js/cg-tray.fasl")
(#x94345bf0e9cbd17ad18f4984c6f1bf18 . "./code.cg2js/cg-plot-widget.fasl")
(#x688d337c799703250e4ec761872a224d . "./code.cg2js/cg-pixmap.fasl")
(#xff7e907f82bd1c21cfdc0f155be458ee . "./code.cg2js/cg-font-dialog.fasl")
(#x9ae8e02099d0351d114d8098716c67ab . "./code.cg2js/cg-os-widget.fasl")
(#x3c303f6de2c1a2e6c3185cbe64b6f6d4 . "./code.cg2js/cg-text-or-combo.fasl")
(#xeda51ac492c56f54d27088c78df0ece2 . "./code.cg2js/cg-trackbar.fasl")
(#xdeb8d092d96bcbcc5d2236924af4cfd1 . "./code.cg2js/cg-up-down-control.fasl")
(#x9b86006c1e91aa1be378f42f348bc3c5 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x513cdddedf3996eb1a5e8e42a0e5582b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7dc10dda5a20bab647a0018eb2aa31a8 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x13b42feac15cf708abcf1f1ea1085a0b . "./code.cg2js/cg-bitmap-stream.fasl")
(#x0e9282332a06471fd414111c9aee79c0 . "./code.cg2js/cg-chart-widget.fasl")
(#x70d91ba2d7835e8f1959ebc77ad28704 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf35ef94e01be377adc88922586aa03bd . "./code.cg2js/cg-group-box.fasl")
(#xa37a93e976ade2fdbeaa278f6c476939 . "./code.cg2js/cg-static-text.fasl")
(#xd9d5b09b957fec7fed34aa549ac53ab7 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xdb031beaf54dc915d1e8c887d08517fd . "./code.cg2js/cg-color-dialog.fasl")
(#xdffe5006ca9755c177015a840f9f0525 . "./code.cg2js/cg-intersect.fasl")
(#xeaa1d4463940f3e6dc1b8acf437b17f3 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xaf77f38beafa5dc6b7316d281c7f973f . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x8bf7e9c01a2be45f21e66af16f7db659 . "./code.cg2js/cg-html-dialog.fasl")
(#x55c9214f027f68823f21c21dda4d291e . "./code.cg2js/cg-lettered-menu.fasl")
(#x2d538e663626f743e572fe0840a85839 . "./code.cg2js/cg-class-grid.fasl")
(#xf14add8e3c396abee8f54077ed7ff761 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x791bf8638169e73e5f0d5ef75675040d . "./code.cg2js/cg-calendar.fasl")
(#x5384ce23e7c08ae738e7b34a125b387b . "./code.cg2js/cg-editable-text.fasl")
(#x287c55b9bef11f775cb85b1f9656f817 . "./code.cg2js/cg-property.fasl")
(#x18297dae8cc90819757095c34826adea . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x23cab959bb8b13bd657cc0d0199561e8 . "./code.cg2js/cg-grid-widget.fasl")
(#x6550cfaaafaa3e0aa7fb54b36706214f . "./code.cg2js/cg-paren-matching.fasl")
(#xdf386f60e9e92f43913a3ba3c8554368 . "./code.cg2js/cg-dev.fasl")
(#x6ca9948da6e86a4d9ceabc7a2dafe730 . "./code.cg2js/cg-drawable.fasl")
(#x409d08c70676af1fe4c18a0c04c505bb . "./code.cg2js/cg-curve.fasl")
(#x718128a39d88eec56d945d4018451c6f . "./code.cg2js/cg-lisp-widget.fasl")
(#xf37685c5a4c4274850cb14fa1cc7ebb2 . "./code.cg2js/cg-dde-utils.fasl")
(#xf3248f99db94978de3d79fc6f84a5cd0 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xf4583ee1bd5cdd758ecf385b95281a8d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x52c847ae84278f8985f3f2ed4b85a766 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa9895778a382e4c0ae82b24c203f26b4 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x09b534c44eb25c2f10c9914aef91bab0 . "./code.cg2js/cg-lamp.fasl")
(#x3b288c15be312fa7669cd46ee8bd1390 . "./code.cg2js/cg-common-control.fasl")
(#x74cb03e06d015f22bb0c70f929cd8699 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x4f7d70b70ca3337888bab2365e82c2fa . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x84d3e84b8b428ea740a9f9cf89566ee4 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x9096a1d9048c61731bac467e881779fd . "./code.cg2js/cg-project-window.fasl")
(#xf0328f9af6fd05e96d7a9ee044e1f6fc . "./code.cg2js/cg-sample-file-menu.fasl")
(#x617df97883dfdf5217d9ccd1b89c7804 . "./code.cg2js/cg-class-support.fasl")
(#x847a96f7d6a6029c30e8d1897866c7fb . "./code.cg2js/cg-multi-picture-button.fasl")
(#xf3663cf93ac366191a6506d5f92ba5c3 . "./code.cg2js/cg-yes-no-list.fasl")
(#x66fb52e3affbd90cc76ce66a950eaa67 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xbe7e445199f6a92fb98ae0ae380c1632 . "./code.cg2js/cg-string-dialog.fasl")
(#x03875b4b9a26cf396b459f4d1c7a5c3f . "./code.cg2js/cg-menu-tooltip.fasl")
(#x686533588ba7b89ea48c24d3889b10a7 . "./code.cg2js/cg-dropping-outline.fasl")
(#x607a29da9d1ee67b6b20868245494bd2 . "./code.cg2js/cg.fasl")
(#x2df9d679127d3b2ff5499c78b8e2b44a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xdf9942fc0255074ba2fdae5f142c7c48 . "./code.cg2js/cg-scaling-stream.fasl")
(#x01635f7fb44c3ab254d19f497466f41b . "./code.cg2js/cg-yes-no-string.fasl")
(#x9c435936b3e8c208822edd5adf4ed57c . "./code.cg2js/cg-icon-pixmap.fasl")
(#x16057949675babe5cbed739ce6f3a95a . "./code.cg2js/cg-web-browser.fasl")
(#x891ae11f10d4804dda71c7af9d4d21db . "./code.cg2js/cg-clipboard-stack.fasl")
(#x78b853ea723eab79a62cecfa45ec4c58 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x44c0d3e37c965c82aa9047a14adcbff5 . "./code.cg2js/cg-message-dialog.fasl")
(#x09fac403ff14b84052bfedafb545de4f . "./code.cg2js/cg-comtab.fasl")
(#x26165363eeaac5802c55c461a7153108 . "./code.cg2js/cg-base.fasl")
(#x82d0a7dbe88cc5dd342e0e4e58929ec4 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x10ccbbc4be3bf1ea35ed6b173c84d396 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2501416cc6c153e994e92dd4cea00ed8 . "./code.cg2js/cg-utility-dialog.fasl")
(#xc6d3749543b5f0f448c683c5a7bb5816 . "./code.cg2js/cg-object-editor.fasl")
(#x139dd2a1fa7f6532108365088ee574c6 . "./code.cg2js/cg-edit-in-place.fasl")
(#x5ccb61eaac584f34c6708b8646039f64 . "./code.cg2js/cg-radio-button.fasl")
(#x330357a4c9fdb1f070f2ef614e9bcb0a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2e1c7619aab40fbcc6a4304d921b0c1e . "./code.cg2js/cg-nodes.fasl")
(#x81e7eb50e7fff768b3b3eb78fe953f3b . "./code.cg2js/cg-get-position.fasl")
(#xd74d5b0305e1b788c910a050e2302be6 . "./code.cg2js/cg-directory-dialog.fasl")
(#x89a2ec5889840ba1d9eb612f928ec812 . "./code.cg2js/cg-lisp-text.fasl")
(#x7b257c1c63ca27ece59aeb633fc2418e . "./code.cg2js/cg-combo-box.fasl")
(#xd34a903c4c34a00946962e7a957fdfc2 . "./code.cg2js/cg-text-widget.fasl")
(#x2c11901baf9d5384c7edd5002e782a3c . "./code.cg2js/cg-list-view.fasl")
(#xa95299ed016bcdfc1848ba45666b61bd . "./code.cg2js/cg-graphics-context.fasl")
(#x578897034a16545225be077762893db8 . "./code.cg2js/cg-get-pixmap.fasl")
(#xba24554d6d7d1dae84606def7722cb97 . "./code.cg2js/cg-menu.fasl")
(#x8f12bd12db1db1b5380684a61ff7b96f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9c0d0ddf00541829db96953c09e2a125 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xd687006b169bf001eb70f59315e59f06 . "./code.cg2js/cg-tooltip.fasl")
(#x940789aba398e72479b3ccf9bc2f8969 . "./code.cg2js/cg-html-widget.fasl")
(#x633a653273ec1774dc66cc834b3f2c92 . "./code.cg2js/cg-choice-list.fasl")
(#x23dd824cc2cfe555002eb87c3f20df16 . "./code.cg2js/cg-icon.fasl")
(#x71af619302f12932c300246f1eeab6a1 . "./code.cg2js/cg-acache.fasl")
(#x8b8ba2102743e5774c1b6705bc5b7a4d . "./code.cg2js/cg-template-string.fasl")
(#x44c9f6306f3d46d7ff09d18de51a8563 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe1662b3fdedaa09d742139050bf35a7d . "./code.cg2js/cg-toggling-widget.fasl")
(#xa36b383d93ec1db0e7db850d7432a19c . "./code.cg2js/cg-tab-control.fasl")
(#x1434542ebdb994666abf82f3e32eec6f . "./code.cg2js/cg-split-bar.fasl")
(#xdb94708cd8698e5a1609dbcf33c2a3e1 . "./code.cg2js/cg-picture-widget.fasl")
(#x8825f9497910ce365d753a64700543d1 . "./code.cg2js/cg-player.fasl")
(#xd52b52f18fd64ee1e85d48d35036a1fb . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x0977ceb697202693109268f3f8f7e805 . "./code.cg2js/cg-choose-printer.fasl")
(#x75e874f82b4e484154dde5e128b02254 . "./code.cg2js/cg-os-window.fasl")
(#x29a33bc863d325a94bfa63dbbe902d79 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf00b8886a5b5594f6d0fb53978db5622 . "./code.cg2js/cg-clipboard.fasl")
(#xdc5a223cd7b88563f4f876d342400cba . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9965957d5979bee4a423d4f2fa0d1f0d . "./code.cg2js/cg-hotspot.fasl")
(#xb31b64ad0ceb51533cfb1d582ce1ede0 . "./code.cg2js/cg-check-box.fasl")
(#xc79d64650f6aa975ad1f94c515c65043 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6f2c44205b58cc1ba5752819b9aaefb0 . "./code.cg2js/cg-file-dialog.fasl")
(#x07dd9a5d6c265684aab0f58890e29cde . "./code.cg2js/cg-toolbar.fasl")
(#x0e071bf33d3bade14f9c04e857fdee21 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xe3b0aee5637c67714f19f4f80763479c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc721989c10bb9b8e387d006ba0e69b16 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x8972fb5d113df2dd5f199bedba6ef61e . "./code.idejs/ide-project.fasl")
(#xc66b290941d383a6e6e7a890d0a9331c . "./code.idejs/ide-intro.fasl")
(#x64eb4d40a25d09a5742988bbf77cdb3f . "./code.idejs/ide-find-in-files.fasl")
(#xab72d4238254f782e3d28671441eb75a . "./code.idejs/ide-grapher.fasl")
(#xe7b1501f0f35337b9a44b910bb77fbd1 . "./code.idejs/ide-inspector.fasl")
(#xe6daaacfcf6b579f14bebe061d71df50 . "./code.idejs/ide-editor.fasl")
(#x06c7c686ed0f0430e6fdd60842a1cafe . "./code.idejs/ide-window-list.fasl")
(#x04a4e7cee272900dcc5b9bcbd1a0340d . "./code.idejs/ide-process-browser.fasl")
(#x890dc4075899e70f3ffc7f0e7d5208ec . "./code.idejs/ide-patch-dialog.fasl")
(#x66a77071926a6ae1f069def684dded2e . "./code.idejs/ide-class-grapher.fasl")
(#xae083ed9234d3d3df4d3439a47787b74 . "./code.idejs/ide-base.fasl")
(#xf599f84674ad6d8f92293b9d3b4eb72f . "./code.idejs/ide.fasl")
(#xa4d861bf8ab902818d09344a9f436c1d . "./code.idejs/ide-trace.fasl")
(#xc5c851b1f399273fe1596a6fef419c06 . "./code.idejs/ide-form.fasl")
(#xc8e1a6d0f03062a8002215bda3987545 . "./code.idejs/ide-find-definitions.fasl")
(#x165ae6106456e9c0f7c9e4803f593f85 . "./code.idejs/ide-apropos.fasl")
(#xc8c5ca26ce48cf38e26ed1a43c7daaa6 . "./code.idejs/ide-class-browser.fasl")
(#xdbc97090fff01c6705dcb1a9d125cea5 . "./code.idejs/ide-navigator.fasl")
(#x84f66ce752702a7a5a0fbcfb520d4018 . "./code.idejs/ide-clipboard.fasl")
(#x75d261c0b2e7c0f7bbba96ace006f8b0 . "./code.idejs/ide-acache.fasl")
(#xe32f2e1824bb862007533501f7578871 . "./code.idejs/ide-keyshort.fasl")
(#x7fd12f9bf383a36f9520b62f971830bb . "./code.idejs/ide-package-list.fasl")
(#x369326274a570c2df2fc689e088af114 . "./code.idejs/ide-dev.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
